import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;


class SwitchPanel extends JFrame implements ActionListener
{
	
	// background hardware
	private Store store;
	private Control control;
	public CrtPanel crtPanel;
	public Baby baby;
	
	// buttons
	public PushButton[] numberKey;
	public ToggleSwitch[] lineSwitch;
	public ToggleSwitch[] functionSwitch;
	ToggleSwitch manAuto;
	ToggleSwitch prePulse;
	
	InterlockingPushButton crSelect;
	InterlockingPushButton accSelect;
	InterlockingPushButton storeSelect;
	ButtonGroup monitorSelectGroup;
	
	KeySwitch klcSwitch;
	KeySwitch kscSwitch;
	KeySwitch kacSwitch;
	KeySwitch kccSwitch;
	
	KeySwitch kspSwitch;
	ToggleSwitch eraseWrite;
	
	public static Color backgroundColor = new Color(206, 205, 201);
	
	public SwitchPanel(Store aStore, Control aControl, CrtPanel aCrtPanel, Baby aBaby)
	{
		store = aStore;
		control = aControl;
		crtPanel = aCrtPanel;
		baby = aBaby;
		
		// set up window
		setTitle("Switch panel");
		setSize(660, 545);
		
		Container contentPane = getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS) );		 
		
		setBackground(backgroundColor);
		contentPane.setBackground(backgroundColor);
		
		// typewriter
		
		JPanel typeWriter = new JPanel();
		typeWriter.setBackground(backgroundColor);
		typeWriter.setLayout(new GridLayout(5,8) );			// 8 across. 5 down
		typeWriter.setBorder( BorderFactory.createLoweredBevelBorder() );
		
		numberKey = new PushButton[40];
		// create 40 keys
		for(int keyNumber=0; keyNumber<40; keyNumber++)
		{
			numberKey[keyNumber] = new PushButton("" + keyNumber, AbstractButton.CENTER);
			// add mouse listeners to keys 0-31
			if(keyNumber < 32)
				numberKey[keyNumber].addMouseListener(new TypeWriterPushed(keyNumber, control, store, this));
		}
		
		// add keys in correct order for display with this layout
		typeWriter.add(numberKey[0]);
		typeWriter.add(numberKey[5]);
		typeWriter.add(numberKey[10]);
		typeWriter.add(numberKey[15]);
		typeWriter.add(numberKey[20]);
		typeWriter.add(numberKey[25]);
		typeWriter.add(numberKey[30]);
		typeWriter.add(numberKey[35]);
		
		typeWriter.add(numberKey[1]);
		typeWriter.add(numberKey[6]);
		typeWriter.add(numberKey[11]);
		typeWriter.add(numberKey[16]);
		typeWriter.add(numberKey[21]);
		typeWriter.add(numberKey[26]);
		typeWriter.add(numberKey[31]);
		typeWriter.add(numberKey[36]);
		
		typeWriter.add(numberKey[2]);
		typeWriter.add(numberKey[7]);
		typeWriter.add(numberKey[12]);
		typeWriter.add(numberKey[17]);
		typeWriter.add(numberKey[22]);
		typeWriter.add(numberKey[27]);
		typeWriter.add(numberKey[32]);
		typeWriter.add(numberKey[37]);
		
		typeWriter.add(numberKey[3]);
		typeWriter.add(numberKey[8]);
		typeWriter.add(numberKey[13]);
		typeWriter.add(numberKey[18]);
		typeWriter.add(numberKey[23]);
		typeWriter.add(numberKey[28]);
		typeWriter.add(numberKey[33]);
		typeWriter.add(numberKey[38]);
		
		typeWriter.add(numberKey[4]);
		typeWriter.add(numberKey[9]);
		typeWriter.add(numberKey[14]);
		typeWriter.add(numberKey[19]);
		typeWriter.add(numberKey[24]);
		typeWriter.add(numberKey[29]);
		typeWriter.add(numberKey[34]);
		typeWriter.add(numberKey[39]);
		
		
		// staticisor
		
		JPanel staticisor = new JPanel( new BorderLayout() );
		staticisor.setBackground(backgroundColor);
		staticisor.setBorder( BorderFactory.createLoweredBevelBorder() );
		
		JPanel staticisorTop = new JPanel();
		JPanel staticisorBottom = new JPanel();
		
		JPanel linePanel = new JPanel();
		linePanel.setBackground(backgroundColor);
		lineSwitch = new ToggleSwitch[5];
		ActionListener lineSwitchActionListener = new LineSwitchActionListener();
		for(int switchNumber=0; switchNumber<5; switchNumber++)
		{
			lineSwitch[switchNumber] = new ToggleSwitch("" + switchNumber, AbstractButton.CENTER, AbstractButton.NORTH);
			linePanel.add(lineSwitch[switchNumber]);
			lineSwitch[switchNumber].addActionListener(lineSwitchActionListener);
		}
		
				
				
		JPanel functionPanel = new JPanel();
		functionPanel.setBackground(backgroundColor);
		functionSwitch = new ToggleSwitch[3];
		for(int switchNumber=0; switchNumber<3; switchNumber++)
		{
			functionSwitch[switchNumber] = new ToggleSwitch("" + (13 + switchNumber), AbstractButton.CENTER, AbstractButton.NORTH);
			functionPanel.add(functionSwitch[switchNumber]);
		}
		
		manAuto = new ToggleSwitch("STAT", AbstractButton.CENTER, AbstractButton.NORTH);
		manAuto.addActionListener( new ManAutoPressed() );
		JPanel statLabelPanel = new JPanel( new BorderLayout() );
		JLabel manLabel = new JLabel("MAN");
		JLabel spacingLabel = new JLabel(" ");
		JLabel autoLabel = new JLabel("AUTO");
		manLabel.setForeground(Color.black);
		autoLabel.setForeground(Color.black);
		statLabelPanel.add(manLabel, BorderLayout.NORTH);
		statLabelPanel.add(spacingLabel, BorderLayout.CENTER);
		statLabelPanel.add(autoLabel, BorderLayout.SOUTH);
		
		
		staticisorTop.add(linePanel);
		
		staticisorBottom.add(functionPanel);
		staticisorBottom.add( new JLabel("     ") );
		staticisorBottom.add( new JLabel("     ") );
		staticisorBottom.add( new JLabel("     ") );
		staticisorBottom.add(manAuto);			// up is man, down is auto
		staticisorBottom.add(statLabelPanel);
		
		staticisor.add(staticisorTop, BorderLayout.NORTH);
		staticisor.add(staticisorBottom, BorderLayout.SOUTH);
		
		// display controls
		
		JPanel displayControls = new JPanel();
		displayControls.setBackground(backgroundColor);
		displayControls.setBorder( BorderFactory.createLoweredBevelBorder() );
		
		// note, CS switch == prePulse switch
		JPanel csSwitchPanel = new JPanel();
		csSwitchPanel.setLayout( new BoxLayout(csSwitchPanel, BoxLayout.Y_AXIS) );
		JLabel csStopLabel = new JLabel("STOP");
		csStopLabel.setForeground(Color.black);
		prePulse = new ToggleSwitch("CS", AbstractButton.LEFT, AbstractButton.NORTH);
		prePulse.addActionListener( new PrePulsePushed() );
		csSwitchPanel.add(csStopLabel);
		csSwitchPanel.add(prePulse);
		
		JPanel monitorSelector = new JPanel();
		monitorSelector.setBackground(backgroundColor);
		
		crSelect = new InterlockingPushButton("C", AbstractButton.NORTH);
		accSelect = new InterlockingPushButton("A", AbstractButton.NORTH);
		storeSelect = new InterlockingPushButton("So", AbstractButton.NORTH);
		monitorSelectGroup = new ButtonGroup();
		monitorSelectGroup.add(crSelect);
		monitorSelectGroup.add(accSelect);
		monitorSelectGroup.add(storeSelect);
		monitorSelector.add(crSelect);
		monitorSelector.add(accSelect);
		monitorSelector.add(storeSelect);
		crSelect.addActionListener(new DisplaySelectPressed(CrtPanel.CONTROL));
		accSelect.addActionListener(new DisplaySelectPressed(CrtPanel.ACCUMULATOR));
		storeSelect.addActionListener(new DisplaySelectPressed(CrtPanel.STORE));		
		
		// storageClearingKeys
		// to be added to displayControls panel
	
		JPanel storageClearingKeys = new JPanel();
		storageClearingKeys.setBackground(Color.black);
		
		kspSwitch = new KeySwitch("KC", "black_key_switch_up.gif", "black_key_switch_down.gif");		// KC == KSP
		kspSwitch.addActionListener(new KspPushed() );
		klcSwitch = new KeySwitch("KLC", "grey_key_switch_up.gif", "grey_key_switch_down.gif");
		klcSwitch.addMouseListener(new KlcPressed() );
		kscSwitch = new KeySwitch("KSC", "red_key_switch_up.gif", "red_key_switch_down.gif");
		kscSwitch.addMouseListener(new KscPushed() );
		kacSwitch = new KeySwitch("KAC", "grey_key_switch_up.gif", "grey_key_switch_down.gif");
		kacSwitch.addMouseListener(new KacPushed() );
		kccSwitch = new KeySwitch("KCC", "grey_key_switch_up.gif", "grey_key_switch_down.gif");
		kccSwitch.addMouseListener(new KccPushed() );
		
		
		KeySwitch kbcSwitch = new KeySwitch("KBC", "grey_key_switch_up.gif", "grey_key_switch_down.gif");		// unconnected buttons
		KeySwitch kecSwitch = new KeySwitch("KEC", "red_key_switch_up.gif", "red_key_switch_down.gif");
		KeySwitch kmcSwitch = new KeySwitch("KMC", "red_key_switch_up.gif", "red_key_switch_down.gif");
		
		kspSwitch.setForeground(Color.white);
		klcSwitch.setForeground(Color.white);
		kscSwitch.setForeground(Color.white);
		kacSwitch.setForeground(Color.white);
		kbcSwitch.setForeground(Color.white);
		kccSwitch.setForeground(Color.white);
		kecSwitch.setForeground(Color.white);
		kmcSwitch.setForeground(Color.white);
		
		storageClearingKeys.add(kspSwitch);
		storageClearingKeys.add(klcSwitch);
		storageClearingKeys.add(kscSwitch);
		storageClearingKeys.add(kacSwitch);
		storageClearingKeys.add(kbcSwitch);
		storageClearingKeys.add(kccSwitch);
		storageClearingKeys.add(kecSwitch);
		storageClearingKeys.add(kmcSwitch);
		
		JPanel eraseWritePanel = new JPanel( new BorderLayout() );
		eraseWrite = new ToggleSwitch("ERASE", AbstractButton.CENTER, AbstractButton.NORTH);
		JLabel writeLabel = new JLabel("WRITE");
		writeLabel.setForeground(Color.black);
		eraseWritePanel.add(eraseWrite, BorderLayout.CENTER);
		eraseWritePanel.add(writeLabel, BorderLayout.SOUTH);
		
		// set up panel with monitor select switches above and CS switch below
		JPanel monitorSelectorAndCS = new JPanel();
		monitorSelectorAndCS.setLayout(new BoxLayout(monitorSelectorAndCS, BoxLayout.Y_AXIS) );
		monitorSelectorAndCS.add(monitorSelector);		// containing prePulse and it's 'STOP' label
		monitorSelectorAndCS.add(csSwitchPanel);
		
		displayControls.add(monitorSelectorAndCS);
		displayControls.add(storageClearingKeys);
		displayControls.add(eraseWritePanel);
						
		// add each panel to main panel
		
		contentPane.add(typeWriter);
		contentPane.add(staticisor);
		contentPane.add(displayControls);
		
		// add tool tips
		for(int keyNumber = 0; keyNumber < 32; keyNumber++)
			numberKey[keyNumber].setToolTipText("Typewriter adjusts bit " + keyNumber + " of the action line.");
		for(int keyNumber = 32; keyNumber < 40; keyNumber++)
			numberKey[keyNumber].setToolTipText("Left unconnected.");
		for(int x=0; x<5; x++)
			lineSwitch[x].setToolTipText("Selects the action line to be adjusted or executed.");
		for(int x=0; x<3; x++)
			functionSwitch[x].setToolTipText("Selects the function number to be manually executed.");
		manAuto.setToolTipText("Select whether to execute the store or the manual instruction.");
		prePulse.setToolTipText("If switched down then continually executes instructions.");
		klcSwitch.setToolTipText("Clears the current action line of the store.");
		kscSwitch.setToolTipText("Clears the entire store.");
		kccSwitch.setToolTipText("Clears the CI, PI and Accumulator.");
		kspSwitch.setToolTipText("Executes a single instruction.");
		eraseWrite.setToolTipText("Selects whether typewriter erases or writes bits.");
		accSelect.setToolTipText("Displays the accumulator on the monitor.");
		crSelect.setToolTipText("Displays the control on the monitor");
		storeSelect.setToolTipText("Displays the store on the monitor.");
		kacSwitch.setToolTipText("Clears the accumulator");
		
		kbcSwitch.setToolTipText("Not connected");
		kecSwitch.setToolTipText("Not connected");
		kmcSwitch.setToolTipText("Not connected");
		
		// set up default settings
		storeSelect.setSelected(true);	// default to display store on monitor
		eraseWrite.setSelected(true);	// default to write
		setManAuto(true);				// default to auto
		// set L stat switches to all be on
		for(int lStatSwitch=0; lStatSwitch<5; lStatSwitch++)
			lineSwitch[lStatSwitch].setSelected(true);
		// likewise F stat switches
		for(int fStatSwitch=0; fStatSwitch<3; fStatSwitch++)
			functionSwitch[fStatSwitch].setSelected(true);
	}
	
	
	public void actionPerformed(ActionEvent e)
	{
	}
	
	// go through all line switches and return int of line selected
	public int getLineValue()
	{
		int result = 0;
		for(int switchNumber = 0; switchNumber<5; switchNumber++)
		{
			if( lineSwitch[switchNumber].isSelected() )
				result += (1<<switchNumber);
		}
		return result;
	}
	
	// go through all function switches and return int of function selected
	public int getFunctionValue()
	{
		int result = 0;
		for(int switchNumber = 0; switchNumber<3; switchNumber++)
		{
			if(functionSwitch[switchNumber].isSelected())
				result += (1<<switchNumber);
		}
		return result;
	}
	
	// return executable value of the line and function switches
	// (as would be represented if taken from store)
	public synchronized int getLineAndFunctionValue()
	{
		int value = getLineValue() | (getFunctionValue() << 13);
		return value;
	}
	
	public void setEraseWrite(boolean value)
	{
		eraseWrite.setSelected(value);
	}
	
	public boolean getEraseWrite()
	{
		return eraseWrite.isSelected();
	}
	
	// true is pulse, false is pre
	public boolean getPrePulse()
	{
		return prePulse.isSelected();
	}
	
	// true is pulse, false is pre
	public void setPrePulse(boolean value)
	{
		prePulse.setSelected(value);
	}
	
	// true is auto, false is man
	public boolean getManAuto()
	{
		return manAuto.isSelected();
	}
	
	// true is auto, false is man
	public void setManAuto(boolean value)
	{
		manAuto.setSelected(value);
	}
	
	public void redrawCrtPanel()
	{
		crtPanel.render();
		crtPanel.repaint();
	}
	
	public void updateActionLine()
	{
		// if automatic
		if( getManAuto() )
		{
			crtPanel.setActionLine( control.getLineNumber( control.getControlInstruction() ) );
		}
		// else manual
		else
		{
			crtPanel.setActionLine( getLineValue() );
		}
	}
	
	
	////////////////////////////////////////////////////////////
	// START OF EVENT HANDLING INNER CLASSES
	////////////////////////////////////////////////////////////
	
	// if pulse execute instructions repeatedly either from store or from line and function switches
	class PrePulsePushed implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			// are we now on pre or pulse?
			// on pulse
			if( getPrePulse() )
			{
				// turn off the highlighted action line
				crtPanel.setActionLine(-1);				
				// whether to take instructions from store or line and function switches
				// is handled in the actual execution of the animation
				baby.startAnimation();
			}
			// on pre
			else
			{
				baby.stopAnimation();
				updateActionLine();
			}
		}
	}
	
	// change selected display
	class DisplaySelectPressed implements ActionListener
	{
		int displayValue;
		
		public DisplaySelectPressed(int value)
		{
			displayValue = value;
		}
		
		public void actionPerformed(ActionEvent e)
		{
			crtPanel.setCrtDisplay(displayValue);
		}
	}
	
	// clear store
	class KscPushed implements MouseListener
	{
		public void mousePressed(MouseEvent e)
		{
			if(Baby.running)
			{
				control.setKscPressed(true);
			}
			else
			{
				store.reset();
				redrawCrtPanel();
			}
		}
		
		public void mouseReleased(MouseEvent e)
		{
			control.setKscPressed(false);
		}
		
		public void mouseEntered(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { }
		public void mouseExited(MouseEvent e) {	}
	}
	
	// clear CI,PI and ACC
	class KccPushed implements MouseListener
	{
		public void mousePressed(MouseEvent e)
		{
			if(Baby.running)
			{
				control.setKccPressed(true);
			}
			else
			{
				control.setControlInstruction(0);
				control.setPresentInstruction(0);
				control.setAccumulator(0);
				updateActionLine();
				redrawCrtPanel();
			}
		}
		
		public void mouseReleased(MouseEvent e)
		{
			control.setKccPressed(false);
		}
		
		public void mouseEntered(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { }
		public void mouseExited(MouseEvent e) {	}
	}
	
	// execute a single instruction (either from store or from line and function switches)
	class KspPushed implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			// clear stop flag if set
			if(control.getStopFlag() )
				control.setStopFlag(false);
			
			// true is Auto, false is Man
			if(getManAuto() )
			{
				// perform single instruction from store
				control.executeAutomatic();
				updateActionLine();
				crtPanel.render();
				crtPanel.repaint();
			}
			else
			{
				// perform single instruction from line and function switches
				control.executeManual();
				updateActionLine();
				crtPanel.render();
				crtPanel.repaint();
			}
		}
	}

	// typewriter button pushed
	class TypeWriterPushed implements MouseListener
	{
		private int keyNumber;
		private Control control;
		private SwitchPanel switchPanel;
		private Store store;
		
		public TypeWriterPushed(int aKeyNumber, Control aControl, Store aStore, SwitchPanel aSwitchPanel)
		{
			keyNumber = aKeyNumber;
			control = aControl;
			switchPanel = aSwitchPanel;
			store = aStore;
		}
		
		public void mousePressed(MouseEvent e)
		{
			// if running then adjust every action line
			if(Baby.running)
			{
				// signal control that a key is pressed
				// arguments: key pressed, number of key pressed, is it a write?
				control.setKeyPressed(true, keyNumber);
				
				// note, m1sim incorrectly stops running in this case rather than corrupting store lines
			}
			// otherwise just do current action line
			else
			{
				int lineNumber = switchPanel.getLineValue();
				// true = write, false = erase
				if(switchPanel.getEraseWrite() )
				{
					store.setLine( lineNumber, store.getLine(lineNumber) | (1<<keyNumber) );
				}
				else
				{
					store.setLine( lineNumber, store.getLine(lineNumber) & (~(1<<keyNumber)) );
				}
				switchPanel.redrawCrtPanel();
			}
		}
	
		public void mouseReleased(MouseEvent e)
		{
			// notify that the key is no longer pressed whether Baby is still
			// running or not.
			control.setKeyPressed(false, keyNumber );
		}
		
		public void mouseEntered(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { }
		public void mouseExited(MouseEvent e) {	}
	}

	class KlcPressed implements MouseListener
	{
		public void mousePressed(MouseEvent e)
		{
			if(Baby.running)
			{
				control.setKlcPressed(true);
			}
			else
			{
				store.setLine( getLineValue(), 0 );
				crtPanel.render();
				crtPanel.repaint();
			}
		}
		
		public void mouseReleased(MouseEvent e)
		{
			control.setKlcPressed(false);
		}
	
		public void mouseEntered(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { }
		public void mouseExited(MouseEvent e) {	}
	}
	
	class KacPushed implements MouseListener
	{
		public void mousePressed(MouseEvent e)
		{
			if(!Baby.running)
			{
				control.setAccumulator( 0 );
				crtPanel.render();
				crtPanel.repaint();
			}
			else
			{
				control.setKacPressed(true);
			}
		}
		
		public void mouseReleased(MouseEvent e)
		{
			control.setKacPressed(false);
		}
	
		public void mouseEntered(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { }
		public void mouseExited(MouseEvent e) {	}
	}
	
	class LineSwitchActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			updateActionLine();
		}
	}

	class ManAutoPressed implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			updateActionLine();
		}
	}

}